@@ -1,3 +1,4 @@
+#![macro_use]
 use crate::addressing::Address;
 use crate::hash::{decode, hash, Hash, Hashable};
 use crate::models;
@@ -41,6 +42,35 @@ lazy_static! {
     pub static ref TYPE_ADDR: Address = TYPE_INVARIANT.entity().unwrap();
 }
 
+#[macro_use]
+mod macros {
+    macro_rules! upend_insert_val {
+        ($db_connection:expr, $entity:expr, $attribute:expr, $value:expr) => {{
+            insert_entry(
+                $db_connection,
+                Entry {
+                    entity: $entity.clone(),
+                    attribute: String::from($attribute),
+                    value: EntryValue::Value(serde_json::Value::from($value)),
+                },
+            )?;
+        }};
+    }
+
+    macro_rules! upend_insert_addr {
+        ($db_connection:expr, $entity:expr, $attribute:expr, $addr:expr) => {{
+            insert_entry(
+                $db_connection,
+                Entry {
+                    entity: $entity.clone(),
+                    attribute: String::from($attribute),
+                    value: EntryValue::Address($addr.clone()),
+                },
+            )?;
+        }};
+    }
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Entry {
     pub entity: Address,
@@ -741,22 +771,7 @@ pub fn open_upend<P: AsRef<Path>>(
 
 fn initialize_types(pool: &DbPool) -> Result<()> {
     insert_entry(&pool.get()?, Entry::try_from(&*TYPE_INVARIANT)?)?;
-    insert_entry(
-        &pool.get()?,
-        Entry {
-            entity: TYPE_ADDR.clone(),
-            attribute: String::from(IS_OF_TYPE_ATTR),
-            value: EntryValue::Address(TYPE_ADDR.clone()),
-        },
-    )?;
-    insert_entry(
-        &pool.get()?,
-        Entry {
-            entity: TYPE_ADDR.clone(),
-            attribute: String::from(TYPE_ID_ATTR),
-            value: EntryValue::Value(serde_json::Value::from(TYPE_IS_ATTR)),
-        },
-    )?;
-
+    upend_insert_addr!(&pool.get()?, TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
+    upend_insert_val!(&pool.get()?, TYPE_ADDR, TYPE_ID_ATTR, TYPE_IS_ATTR);
     Ok(())
 }
